/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.ee;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpSession;

import woolpack.utils.KeyIteratorMap;


/**
 * {@link HttpSession}のセッションスコープを{@link Map}に変換するアダプタ。
 * 本クラスは Servlet API のライブラリを使用する。
 * 適用しているパターン：Adapter。
 * @author nakamura
 *
 */
public class HttpSessionMap extends KeyIteratorMap<String,Object> {
	
	private final HttpSession child;
	
	/**
	 * コンストラクタ。
	 * @param child 委譲先。
	 * @throws NullPointerException 引数が null の場合。
	 */
	public HttpSessionMap(final HttpSession child){
		child.getClass();
		this.child = child;
	}

	@Override protected Iterator<String> getKeyIterator() {
		final Enumeration e = child.getAttributeNames();
		return new Iterator<String>(){
			Object key = null;
			public void remove() {
				HttpSessionMap.this.remove(key);
			}
			public boolean hasNext() {
				return e.hasMoreElements();
			}
			public String next() {
				key = e.nextElement();
				return (String)key;
			}
		};
	}

	@Override protected Object getValue(final Object key) {
		return child.getAttribute((String)key);
	}
	
	@Override public Object get(final Object key){
		return getValue((String)key);
	}
	
	@Override public Object put(final String key, final Object value){
		final Object before = getValue(key);
		child.setAttribute(key, value);
		return before;
	}
	
	@Override public Object remove(final Object key){
		final Object before = getValue(key);
		child.removeAttribute((String)key);
		return before;
	}
}
