/*
 * Copyright 2006 Takahiro Nakamura.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package woolpack.text;

import java.text.FieldPosition;
import java.text.Format;
import java.text.ParsePosition;

/**
 * {@link Object#toString()}を使用してフォーマットする変換器。
 * {@link #parseObject(String, ParsePosition)}は変換を行わない。
 * @author nakamura
 *
 */
public class ToStringFormat extends Format {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	/**
	 * コンストラクタ。
	 *
	 */
	public ToStringFormat(){
	}

	@Override
	public StringBuffer format(final Object obj, final StringBuffer toAppendTo,
			final FieldPosition pos) {
		final int start = toAppendTo.length();
		toAppendTo.append(obj.toString());
		pos.setBeginIndex(start);
		pos.setEndIndex(toAppendTo.length());
		return toAppendTo;
	}

	@Override
	public Object parseObject(final String source, final ParsePosition pos) {
		final int i = pos.getIndex();
		pos.setIndex(source.length());
		return source.substring(i);
	}

	@Override public Object clone(){
		return new ToStringFormat();
	}
}
